@@ -2,6 +2,8 @@
 
 import md5, os, stat
 
+from pyutil import mathutil
+
 READSIZE=8192
 
 def md5it(f):
@@ -12,7 +14,7 @@ def md5it(f):
         d = f.read(READSIZE)
     return h.hexdigest()
 
-def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
+def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS, next_multiple=mathutil.next_multiple):
     assert not (HARDLINKEM and DELETEEM) # at most one of these
     # d1 = { k: size, v: { k: (dev, ino,), v: [ fname ] } }
     d1 = {}
@@ -20,7 +22,7 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
     visiteddirs = set() # to avoid symlink recursion
 
     # First pass: build map from size, dev, inode number to filenames, and if PROFILES then print out the first 4 chars of the md5sum and size of each file.
-    def visit(dirpath, dirnames, fnames):
+    def visit(dirpath, dirnames, fnames, next_multiple=next_multiple):
         if dirpath in visiteddirs:
             return
         visiteddirs.add(dirpath)
@@ -40,8 +42,6 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
                     else:
                         l = []
                         d2[(thestat.st_dev, thestat.st_ino,)] = l
-                        if PROFILES:
-                            print "%s:%s" % (md5it(open(fullfname, 'r'))[:4], thestat.st_size)
                     l.append(fullfname)
             except EnvironmentError, le:
                 if VERBOSITY:
@@ -64,14 +64,14 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
     d3 = {}
 
     for (size, d2,) in d1.iteritems():
-        if len(d2) > 1:
+        if (len(d2) > 1) or (len(d2) == 1 and PROFILES):
             if VERBOSITY:
                 print "files with same sizes.  size: %s, fnames: %s" % (size, d2,)
             for ((dev, ino,), fnames,) in d2.iteritems():
                 md5h = None
                 for fname in fnames:
                     try:
-                        md5h = md5it(open(fname, "r")) # Here's the expensive part.
+                        md5h = md5it(open(fname, "rb")) # Here's the expensive part.
                         break
                     except EnvironmentError, le:
                         print "note: couldn't read file named %s because %s" % (fname, le)
@@ -80,6 +80,9 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
                     print "note: couldn't read any of these (same-inode) files: %s" % (fnames,)
                     break
 
+                if PROFILES:
+                    print "%s:%s" % (md5h[:4], next_multiple(size, 4096))
+
                 d4 = d3.setdefault(md5h, {})
                 assert not d4.has_key((dev, ino,))
                 d4[(dev, ino,)] = fnames
@@ -89,7 +92,7 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
                         onename = d4.items()[0][1][0]
                     for ((iterdev, iterino,), iterfnames,) in d4.iteritems():
                         if PROFILES:
-                            print "identical; md5: %s, dev: %s, ino: %s, size: %s" % (md5it(open(onename, 'r'))[:2], iterdev, iterino, size,)
+                            print "identical; md5: %s, dev: %s, ino: %s, size: %s" % (md5h[:4], iterdev, iterino, next_multiple(size, 4096),)
                         else:
                             print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, iterdev, iterino, size, iterfnames,)
                         if HARDLINKEM or DELETEEM:
@@ -104,7 +107,7 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES, DIRS):
                 elif len(d4) > 2:
                     # More collisions.  Print out the new one.
                     if PROFILES:
-                        print "identical; md5: %s, dev: %s, ino: %s, size: %s" % (md5it(open(fnames[0], 'r'))[:2], dev, ino, size,)
+                        print "identical; md5: %s, dev: %s, ino: %s, size: %s" % (md5h[:4], dev, ino, next_multiple(size, 4096),)
                     else:
                         print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, dev, ino, size, fnames,)
                     if HARDLINKEM or DELETEEM:
